import math

from .exception import SVGeometryException

__all__ = [
    'volume_cylinder',
    'volume_cone',
    'volume_frustum',
]

def volume_cylinder(h, r):
    '''Return the volume of the cylinder.

    Arguments:
        h (float): Height of the cylinder.
        r (float): Radius of the cylinder.

    Returns:
        float: The volume of the cylinder.

    Raises:
        SVGeometryException: If any argument is nonpositive.
    '''

    if h <= 0:
        raise SVGeometryException('Height must be positive')

    if r <= 0:
        raise SVGeometryException('Radius must be positive')

    return math.pi * r * r * h

def volume_cone(h, r):
    '''Return the volume of the cone.

    Arguments:
        h (float): Height of the cone.
        r (float): Radius of the cone.

    Returns:
        float: The volume of the cone.

    Raises:
        SVGeometryException: If any argument is nonpositive.
    '''

    if h <= 0:
        raise SVGeometryException('Height must be positive')

    if r <= 0:
        raise SVGeometryException('Radius must be positive')

    return volume_cylinder(h, r) / 3

def volume_frustum(h, r, R):
    '''Return the volume of the frustum.

    It is expected to be a right frustum, i.e. its apex is at its center.

    Arguments:
        h (float): Height of the frustum.
        r (float): Upper radius of the frumstum.
        R (float): Lower radius of the frumstum.

    Returns:
        float: The volume of the frustum.

    Raises:
        SVGeometryException: If any argument is nonpositive.
    '''

    if h <= 0:
        raise SVGeometryException('Height must be positive')

    if r <= 0:
        raise SVGeometryException('Lower radius must be positive')

    if R <= 0:
        raise SVGeometryException('Upper radius must be positive')

    return math.pi * h / 3 * (r*r + R*R + r*R)



from .exception import SVGeometryException

__all__ = [
    'split_trapezium'
]

def split_trapezium(R, r, H, h):
    '''Split a trapezium in two halves.

    It is expected to be a right trapezium, i.e. its apex is at its center.

    Arguments:
        R (float): Upper radius of the trapezium.
        r (float): Lower radius of the trapezium.
        H (float): Height of the trapezium.
        h (float): Height where the trapezium must be splitted.

    Returns:
        dict: A dict with two keys, "upper" and "lower", each one containing
        another dict with keys "upper_radius", "lower_radius" and "H". The
        "upper" dict describes the property of the upper half of the trapezium,
        whilst "lower" describes the lower half.

    Raises:
        SVGeometryException: If any argument is nonpositive or if "h"
        is greater than "H".
    '''

    if r <= 0:
        raise SVGeometryException('"r" must be greater than 0')

    if R <= 0:
        raise SVGeometryException('"R" must be greater than 0')

    if H <= 0:
        raise SVGeometryException('"H" must be greater than 0')

    if h <= 0:
        raise SVGeometryException('"h" must be greater than 0')

    if H < h:
        raise SVGeometryException('"H" must be greater than "h"')

    x1 = R
    y1 = 0.0
    x2 = r
    y2 = H

    a = (y2 - y1) / (x2 - x1)
    b = -a*x1 + y1

    r_apos = ((h - b) / a)

    return {
        'upper': { 'upper_radius': r, 'lower_radius': r_apos, 'H': H - h },
        'lower': { 'upper_radius': r_apos, 'lower_radius': R, 'H': h },
    }



import pathlib

from setuptools import setup

from sv_geometry import __version__ as version

HERE = pathlib.Path(__file__).parent

README = (HERE / 'README.md').read_text()

setup(
    name='sv_geometry',
    version=version,
    description='A suite of handy geometry-related functions',
    long_description=README,
    long_description_content_type='text/markdown',
    url='https://github.com/ihavenonickname/sv-geometry',
    author='Gabriel Blank Stift Mousquer',
    author_email='gabrielblanksm@gmail.com',
    license='MIT',
    packages=['sv_geometry'],
    include_package_data=True,
)
